@@ -5,8 +5,8 @@
 ;; Author:   Dmitry Gutov <dmitry@gutov.dev>
 ;; URL:      https://github.com/dgutov/diff-hl
 ;; Keywords: vc, diff
-;; Package-Version: 20251024.2243
-;; Package-Revision: b2a57ac4fe7a
+;; Package-Version: 20251112.135
+;; Package-Revision: 4548d29eab30
 ;; Package-Requires: ((cl-lib "0.2") (emacs "26.1"))
 
 ;; This file is part of GNU Emacs.
@@ -70,7 +70,18 @@
   (require 'vc-git)
   (require 'vc-hg)
   (require 'face-remap)
-  (declare-function smartrep-define-key 'smartrep))
+  (require 'project))
+
+(defmacro static-if (condition then-form &rest else-forms) ; since Emacs 30.1
+  "A conditional compilation macro.
+Evaluate CONDITION at macro-expansion time.  If it is non-nil,
+expand the macro to THEN-FORM.  Otherwise expand it to ELSE-FORMS
+enclosed in a `progn' form.  ELSE-FORMS may be empty."
+  (declare (indent 2)
+           (debug (sexp sexp &rest sexp)))
+  (if (eval condition lexical-binding)
+      then-form
+    (cons 'progn else-forms)))
 
 (defgroup diff-hl nil
   "VC diff highlighting on the side of a window"
@@ -383,35 +394,37 @@ It can be a relative expression as well, such as \"HEAD^\" with Git, or
     (ignored 'diff-hl-bmp-i)
     (t (intern (format "diff-hl-bmp-%s" type)))))
 
-(defvar vc-svn-diff-switches)
-(defvar vc-fossil-diff-switches)
-(defvar vc-jj-diff-switches)
-
 (defmacro diff-hl-with-diff-switches (body)
-  `(let ((vc-git-diff-switches
-          ;; https://github.com/dgutov/diff-hl/issues/67
-          (cons "-U0"
-                ;; https://github.com/dgutov/diff-hl/issues/9
-                (and (boundp 'vc-git-diff-switches)
-                     (listp vc-git-diff-switches)
-                     (cl-remove-if-not
-                      (lambda (arg)
-                        (member arg '("--histogram" "--patience" "--minimal" "--textconv")))
-                      vc-git-diff-switches))))
-         (vc-hg-diff-switches nil)
-         (vc-svn-diff-switches nil)
-         (vc-fossil-diff-switches '("-c" "0"))
-         (vc-jj-diff-switches '("--git" "--context=0"))
-         (vc-diff-switches '("-U0"))
-         ,@(when (boundp 'vc-disable-async-diff)
-             '((vc-disable-async-diff t))))
-     ,body))
+  `(progn
+     (defvar vc-svn-diff-switches)
+     (defvar vc-fossil-diff-switches)
+     (defvar vc-jj-diff-switches)
+     (let ((vc-git-diff-switches
+            ;; https://github.com/dgutov/diff-hl/issues/67
+            (cons "-U0"
+                  ;; https://github.com/dgutov/diff-hl/issues/9
+                  (and (boundp 'vc-git-diff-switches)
+                       (listp vc-git-diff-switches)
+                       (cl-remove-if-not
+                        (lambda (arg)
+                          (member arg '("--histogram" "--patience" "--minimal" "--textconv")))
+                        vc-git-diff-switches))))
+           (vc-hg-diff-switches nil)
+           (vc-svn-diff-switches nil)
+           (vc-fossil-diff-switches '("-c" "0"))
+           (vc-jj-diff-switches '("--git" "--context=0"))
+           (vc-diff-switches '("-U0"))
+           ,@(when (boundp 'vc-disable-async-diff)
+               '((vc-disable-async-diff t))))
+       ,body)))
 
 (defun diff-hl-modified-p (state)
   (or (memq state '(edited conflict))
       (and (eq state 'up-to-date)
            ;; VC state is stale in after-revert-hook.
-           (or revert-buffer-in-progress-p
+           (or (static-if (>= emacs-major-version 31)
+                   revert-buffer-in-progress
+                 revert-buffer-in-progress-p)
                ;; Diffing against an older revision.
                diff-hl-reference-revision))))
 
@@ -632,6 +645,11 @@ contents as they are (or would be) after applying the changes in NEW."
            "diff-hl--update-safe")))
     (diff-hl--update)))
 
+(defun diff-hl--update-buffer (buf)
+  (when (buffer-live-p buf)
+    (with-current-buffer buf
+      (diff-hl-update))))
+
 (defun diff-hl-with-editor-p (_dir)
   (bound-and-true-p with-editor-mode))
 
@@ -689,14 +707,23 @@ Return a list of line overlays used."
                 (overlay-put h 'insert-behind-hooks hook)))))))
     (nreverse ovls)))
 
+(defun diff-hl--no-query-on-exit (value)
+  (when-let* ((buf (and (stringp value) (get-buffer value)))
+              (proc (get-buffer-process buf)))
+    (set-process-query-on-exit-flag proc nil)))
+
 (defun diff-hl--update ()
   (let* ((orig (current-buffer))
-         (cc (diff-hl-changes)))
+         (cc (diff-hl-changes))
+         (working (assoc-default :working cc))
+         (reference (assoc-default :reference cc)))
+    (diff-hl--no-query-on-exit working)
+    (diff-hl--no-query-on-exit reference)
     (diff-hl--resolve
-     (assoc-default :working cc)
+     working
      (lambda (changes)
        (diff-hl--resolve
-        (assoc-default :reference cc)
+        reference
         (lambda (ref-changes)
           (let ((ref-changes (diff-hl-adjust-changes ref-changes changes))
                 reuse)
@@ -728,14 +755,12 @@ Return a list of line overlays used."
       (dolist (win (get-buffer-window-list))
         (set-window-buffer win (current-buffer))))))
 
-(put 'diff-hl--modified-tick 'permanent-local t)
-
 (defun diff-hl-update-once ()
   ;; Ensure that the update happens once, after all major mode changes.
   ;; That will keep the the local value of <side>-margin-width, if any.
   (unless diff-hl-timer
     (setq diff-hl-timer
-          (run-with-idle-timer 0 nil #'diff-hl-update))))
+          (run-with-idle-timer 0 nil #'diff-hl--update-buffer (current-buffer)))))
 
 (defun diff-hl-add-highlighting (type shape &optional ovl)
   (let ((o (or ovl (make-overlay (point) (point)))))
@@ -1325,6 +1350,7 @@ The value of this variable is a mode line template as in
                                     diff-hl-show-hunk-next))
 
 (when (require 'smartrep nil t)
+  (declare-function smartrep-define-key 'smartrep)
   (let (smart-keys)
     (cl-labels ((scan (map)
                       (map-keymap
@@ -1666,9 +1692,9 @@ effect."
 
 (defun diff-hl--project-root (proj)
   ;; Emacs 26 and 27 don't have `project-root'.
-  (expand-file-name
-   (or (and (fboundp 'project-root) (project-root proj))
-       (project-roots proj))))
+  (expand-file-name (static-if (>= emacs-major-version 28)
+                        (project-root proj)
+                      (project-roots proj))))
 
 (defun diff-hl-set-reference-rev-in-project-internal (rev proj)
   (let* ((root (diff-hl--project-root proj)))
